#!/usr/bin/env python
# -*- coding: utf-8 -*-

import numpy as np
import matplotlib.pyplot as plt
from PIL import Image
import os
import caffe

# 環境変数$CAFFE_DIRを取得
caffe_root = os.environ.get("CAFFE_DIR")

# プロット設定
plt.rcParams['figure.figsize'] = (10, 10)
plt.rcParams['image.interpolation'] = 'nearest'
plt.rcParams['image.cmap'] = 'gray'

# CPUモードに設定
caffe.set_mode_cpu()

# ネットワーク(net)を読み込み、どのようなデータとパラメータがあるのかを出力します
net = caffe.Net(caffe_root + '/examples/net_surgery/conv.prototxt', caffe.TEST)
print("blobs {}\nparams {}".format(net.blobs.keys(), net.params.keys())) # ･･･(1)

# モデルに入力するための画像を1枚読み込みます
im = np.array(Image.open(caffe_root + '/examples/images/cat_gray.jpg'))
plt.title("original image")
plt.imshow(im)
plt.axis('off')
plt.show()

# 読み込んだ画像をCaffe固有の形式(blob)に変更して入力する
im_input = im[np.newaxis, np.newaxis, :, :]
net.blobs['data'].reshape(*im_input.shape)
net.blobs['data'].data[...] = im_input

# 画像をフィルタに通して可視化する関数
def show_filters(net):
    net.forward()
    plt.figure()
    filt_min, filt_max = net.blobs[
        'conv'].data.min(), net.blobs['conv'].data.max()
    for i in range(3):
        plt.subplot(1, 4, i + 2)
        plt.title("filter #{} output".format(i))
        plt.imshow(net.blobs['conv'].data[0, i], vmin=filt_min, vmax=filt_max)
        plt.tight_layout()
        plt.axis('off')
    plt.show()

# 初期状態で画像をフィルタに掛けて可視化
show_filters(net) # 1回目の可視化

# 最初のフィルタの出力を取得
conv0 = net.blobs['conv'].data[0, 0]
print("pre-surgery output mean {:.2f}".format(conv0.mean()))
# 最初のフィルタのバイアスを10に設定
net.params['conv'][1].data[0] = 10.
net.forward()
print("post-surgery output mean {:.2f}".format(conv0.mean()))

show_filters(net) # 2回目の可視化

# カーネルサイズを取得
ksize = net.params['conv'][0].data.shape[2:]
print("ksize: {}".format(ksize))

# ガウシアンフィルタを作成し、畳み込み層の0番目のフィルタとして設定
sigma = 1.
y, x = np.mgrid[-ksize[0] // 2 + 1:ksize[0] //
                2 + 1, -ksize[1] // 2 + 1:ksize[1] // 2 + 1]
g = np.exp(-((x**2 + y**2) / (2.0 * sigma**2)))
gaussian = (g / g.sum()).astype(np.float32)
net.params['conv'][0].data[0] = gaussian

# 水平方向と垂直方向のエッジを検出するsobelフィルタを
# それぞれ畳み込み層の1番目、2番目のフィルタとして設定
net.params['conv'][0].data[1:] = 0.
sobel = np.array(
    (-1, -2, -1, 0, 0, 0, 1, 2, 1), dtype=np.float32).reshape((3,3))
net.params['conv'][0].data[1, 0, 1:-1, 1:-1] = sobel  # 水平方向
net.params['conv'][0].data[2, 0, 1:-1, 1:-1] = sobel.T  # 垂直方向

show_filters(net) # 3回目の可視化
