#!/usr/bin/env python
# -*- coding: utf-8 -*-

import numpy as np
import matplotlib.pyplot as plt
import os
import caffe

# pltの設定を変更します
# 画像サイズを10inch x 10inchに指定します
plt.rcParams['figure.figsize'] = (10, 10)

# 画像の補間処理を無効化します
plt.rcParams['image.interpolation'] = 'none'

# Color mapを明示します
plt.rcParams['image.cmap'] = 'gray'

# 環境変数$CAFFE_DIRを取得
caffe_root = os.environ.get("CAFFE_DIR")

# CPUモードにセット
caffe.set_mode_cpu()

# 学習済みモデルの読み込み
net = caffe.Net(caffe_root + '/models/bvlc_alexnet/deploy.prototxt',
                caffe_root + '/models/bvlc_alexnet/bvlc_alexnet.caffemodel',
                caffe.TEST)

# dataを可視化する関数
def vis_square(data, padsize=1, padval=0):

    # 値が0～1の範囲に収まるように正規化します
    data -= data.min()
    data /= data.max()
    
    # データ数(この場合は特徴マップ数)を基に可視化する画像が正方形になるように計算します
    n = int(np.ceil(np.sqrt(data.shape[0])))
    padding = ((0, n ** 2 - data.shape[0]),
               (0, padsize),
               (0, padsize)) + ((0, 0),
                                ) * (data.ndim - 3)
    data = np.pad(
        data, padding, mode='constant', constant_values=(padval, padval))
    
    # 各データ(この場合は特徴マップ)をタイル状に整列させます
    data = data.reshape(
        (n, n) + data.shape[1:]).transpose((0, 2, 1, 3) + tuple(range(4, data.ndim + 1)))
    data = data.reshape(
        (n * data.shape[1], n * data.shape[3]) + data.shape[4:])
    
    # 表示処理です
    plt.imshow(data)
    plt.show()

# 入力画像を前処理するための準備
transformer = caffe.io.Transformer({'data': net.blobs['data'].data.shape})

# 読み込んだ画像が(height, width, channel)の形式なので、
# channelを一番前の次元へ移動させます。
transformer.set_transpose('data', (2, 0, 1))

# 平均画像の読み込み
transformer.set_mean('data', np.load(
    caffe_root + '/python/caffe/imagenet/ilsvrc_2012_mean.npy').mean(1).mean(1))
# bvlc_alexnet.caffemodelは[0,1]ではなく[0,255]の範囲で値を扱います。
transformer.set_raw_scale('data', 255)
# bvlc_alexnet.caffemodelはRGBではなくBGRで画像を扱います。
transformer.set_channel_swap('data', (2, 1, 0))

# 画像を学習済みモデルの入力し、forward処理を行います。
net.blobs['data'].reshape(1, 3, 227, 227)
net.blobs['data'].data[...] = transformer.preprocess(
    'data', caffe.io.load_image(caffe_root + '/examples/images/cat.jpg'))
out = net.forward()

# net.blobsの形式を確認しています。
print("net.blobsのkeyとvalueのリストを表示")
print([(k, v.data.shape) for k, v in net.blobs.items()])

# conv1が抽出した特徴マップ
features = net.blobs['conv1'].data[0, :36]
vis_square(features, padval=1)

# conv2が抽出した特徴マップ
features = net.blobs['conv2'].data[0, :36]
vis_square(features, padval=1)

# conv3が抽出した特徴マップ
features = net.blobs['conv3'].data[0]
vis_square(features, padval=0.5)

# conv4が抽出した特徴マップ
feat = net.blobs['conv4'].data[0]
vis_square(feat, padval=0.5)

# conv5が抽出した特徴マップ
feat = net.blobs['conv5'].data[0]
vis_square(feat, padval=0.5)

# pool5が抽出した特徴マップ
feat = net.blobs['pool5'].data[0]
vis_square(feat, padval=1)
